"""
Tests for apyori.load_transactions.
"""

# For Python 2 compatibility.
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from nose.tools import eq_

from apyori import load_transactions


def test_empty_data():
    """
    Tests for empty data.
    """
    test_data = StringIO('')
    result = list(load_transactions(test_data))
    eq_(result, [])


def test_empty_string():
    """
    Tests for empty string.
    """
    test_data = StringIO(
        '\n'  # Empty line.
        'A\t\tB\n' # Empty string middle.
        'C\t\n' # Empty string last.
    )
    result = list(load_transactions(test_data))
    eq_(result, [
        [''],
        ['A', '', 'B'],
        ['C', ''],
    ])


def test_normal():
    """
    Tests for normal data.
    """
    test_data = StringIO(
        'A\tB\n' # Normal.
        '"C\t"\r\n' # Quote and Windows line feed code.
        'D' # Final line without line separator.
    )
    result = list(load_transactions(test_data))
    eq_(result, [
        ['A', 'B'], # Normal.
        ['C\t'], # Contains tab.
        ['D'],
    ])



"""
Tests for apyori.dump_as_json.
"""

import json

# For Python 2 compatibility.
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from nose.tools import raises
from nose.tools import eq_

from apyori import RelationRecord
from apyori import OrderedStatistic
from apyori import dump_as_json


def test_normal():
    """
    Test for normal data.
    """
    test_data = RelationRecord(
        frozenset(['A']), 0.5,
        [OrderedStatistic(frozenset([]), frozenset(['A']), 0.8, 1.2)]
    )
    output_file = StringIO()
    dump_as_json(test_data, output_file)

    output_file.seek(0)
    result = json.loads(output_file.read())
    eq_(result, {
        'items': ['A'],
        'support': 0.5,
        'ordered_statistics': [
            {
                'items_base': [],
                'items_add': ["A"],
                'confidence': 0.8,
                'lift': 1.2
            }
        ]
    })


@raises(TypeError)
def test_bad():
    """
    Test for bad data.
    """
    test_data = RelationRecord(
        set(['A']), 0.5,
        [OrderedStatistic(frozenset([]), frozenset(['A']), 0.8, 1.2)]
    )
    output_file = StringIO()
    dump_as_json(test_data, output_file)



"""
Tests for apyori.TransactionManager.
"""

from nose.tools import eq_

from apyori import TransactionManager


def test_empty():
    """
    Test for a empty transaction.
    """
    transactions = []
    manager = TransactionManager(transactions)

    eq_(manager.num_transaction, 0)
    eq_(manager.items, [])
    eq_(manager.initial_candidates(), [])
    eq_(manager.calc_support([]), 1.0)
    eq_(manager.calc_support(['hoge']), 0.0)


def test_normal():
    """
    Test for a normal transaction.
    """
    transactions = [
        ['beer', 'nuts'],
        ['beer', 'cheese'],
    ]
    manager = TransactionManager(transactions)

    eq_(manager.num_transaction, len(transactions))
    eq_(manager.items, ['beer', 'cheese', 'nuts'])
    eq_(manager.initial_candidates(), [
        frozenset(['beer']), frozenset(['cheese']), frozenset(['nuts'])])
    eq_(manager.calc_support([]), 1.0)
    eq_(manager.calc_support(['beer']), 1.0)
    eq_(manager.calc_support(['nuts']), 0.5)
    eq_(manager.calc_support(['butter']), 0.0)
    eq_(manager.calc_support(['beer', 'nuts']), 0.5)
    eq_(manager.calc_support(['beer', 'nuts', 'cheese']), 0.0)


def test_create():
    """
    Test for the factory method.
    """
    transactions = []
    manager1 = TransactionManager.create(transactions)
    manager2 = TransactionManager.create(manager1)
    eq_(manager1, manager2)



"""
Tests for apyori.gen_ordered_statistics.
"""

from mock import Mock
from nose.tools import eq_

from apyori import SupportRecord
from apyori import OrderedStatistic
from apyori import TransactionManager
from apyori import gen_ordered_statistics


def test_normal():
    """
    Test for normal data.
    """
    transaction_manager = Mock(spec=TransactionManager)
    transaction_manager.calc_support.side_effect = lambda key: {
        frozenset(['A']): 0.8,
        frozenset(['B']): 0.4,
        frozenset(['C']): 0.2,
        frozenset(['A', 'B']): 0.2,
        frozenset(['A', 'C']): 0.1,
        frozenset(['B', 'C']): 0.01,
    }.get(key, 0.0)

    test_data = SupportRecord(frozenset(['A', 'B', 'C']), 0.001)
    results = list(gen_ordered_statistics(transaction_manager, test_data))
    eq_(results, [
        OrderedStatistic(
            frozenset(['A', 'B']),
            frozenset(['C']),
            0.001 / 0.2,
            0.001 / 0.2 / 0.2,
        ),
        OrderedStatistic(
            frozenset(['A', 'C']),
            frozenset(['B']),
            0.001 / 0.1,
            0.001 / 0.1 / 0.4,
        ),
        OrderedStatistic(
            frozenset(['B', 'C']),
            frozenset(['A']),
            0.001 / 0.01,
            0.001 / 0.01 / 0.8,
        ),
    ])



"""
Tests for apyori.dump_as_two_item_tsv.
"""

# For Python 2 compatibility.
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from os import linesep
from nose.tools import eq_

from apyori import RelationRecord
from apyori import OrderedStatistic
from apyori import dump_as_two_item_tsv


def test_normal():
    """
    Test for normal data.
    """
    test_data = RelationRecord(
        frozenset(['A', 'B']), 0.5, [
            OrderedStatistic(frozenset(), frozenset(['B']), 0.8, 1.2),
            OrderedStatistic(frozenset(['A']), frozenset(), 0.8, 1.2),
            OrderedStatistic(frozenset(['A']), frozenset(['B']), 0.8, 1.2),
        ]
    )
    output_file = StringIO()
    dump_as_two_item_tsv(test_data, output_file)

    output_file.seek(0)
    result = output_file.read()
    eq_(result, 'A\tB\t0.50000000\t0.80000000\t1.20000000' + linesep)



"""
Tests for apyori.create_next_candidates.
"""

from nose.tools import eq_

from apyori import create_next_candidates


def test_2elem():
    """
    Test for create_next_candidates with 2 elements.
    """
    test_data = [
        frozenset(['A']),
        frozenset(['B']),
        frozenset(['C'])
    ]

    result = create_next_candidates(test_data, 2)
    eq_(result, [
        frozenset(['A', 'B']),
        frozenset(['A', 'C']),
        frozenset(['B', 'C']),
    ])


def test_3elem():
    """
    Test for create_next_candidates with 3 elements.
    """
    test_data = [
        frozenset(['A', 'B']),
        frozenset(['B', 'C']),
        frozenset(['A', 'C']),
        frozenset(['D', 'E']),
        frozenset(['D', 'F']),
    ]

    result = create_next_candidates(test_data, 3)
    eq_(result, [frozenset(['A', 'B', 'C'])])



"""
Tests for apyori.main.
"""

# For Python 2 compatibility.
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from collections import namedtuple
from nose.tools import eq_

from apyori import main


def test_normal():
    """
    Test for normal data.
    """
    delimiter = 'x'
    inputs = ['AxB', 'AxC']
    input_files = [StringIO(inputs[0]), StringIO(inputs[1])]
    input_transactions = [['A', 'B'], ['A', 'C']]
    def load_transactions_mock(input_file, **kwargs):
        """ Mock for apyori.load_transactions. """
        eq_(kwargs['delimiter'], delimiter)
        eq_(next(input_file), inputs[0])
        yield iter(input_transactions[0])
        eq_(next(input_file), inputs[1])
        yield iter(input_transactions[1])

    max_length = 2
    min_support = 0.5
    min_confidence = 0.2
    apriori_results = ['123', '456']
    def apriori_mock(transactions, **kwargs):
        """ Mock for apyori.apriori. """
        eq_(list(next(transactions)), input_transactions[0])
        eq_(list(next(transactions)), input_transactions[1])
        eq_(kwargs['max_length'], max_length)
        eq_(kwargs['min_support'], min_support)
        eq_(kwargs['min_confidence'], min_confidence)
        for result in apriori_results:
            yield result

    def output_func_mock(record, output_file):
        """ Mock for apyori.output_func. """
        output_file.write(record)

    args = namedtuple(
        'ArgumentMock', [
            'input',
            'delimiter',
            'max_length',
            'min_support',
            'min_confidence',
            'output',
            'output_func'
        ]
    )(
        input=input_files, delimiter=delimiter,
        max_length=max_length, min_support=min_support,
        min_confidence=min_confidence, output=StringIO(),
        output_func=output_func_mock
    )
    main(
        _parse_args=lambda _: args,
        _load_transactions=load_transactions_mock,
        _apriori=apriori_mock)
    args.output.seek(0)
    eq_(args.output.read(), ''.join(apriori_results))



"""
Tests for apyori.filter_ordered_statistics.
"""

from nose.tools import eq_

from apyori import OrderedStatistic
from apyori import filter_ordered_statistics


TEST_DATA = [
    OrderedStatistic(frozenset(['A']), frozenset(['B']), 0.1, 0.7),
    OrderedStatistic(frozenset(['A']), frozenset(['B']), 0.3, 0.5),
]


def test_normal():
    """
    Test for normal data.
    """
    result = list(filter_ordered_statistics(
        TEST_DATA, min_confidence=0.1, min_lift=0.5))
    eq_(result, TEST_DATA)


def test_min_confidence():
    """
    Filter by minimum confidence.
    """
    result = list(filter_ordered_statistics(
        TEST_DATA, min_confidence=0.2, min_lift=0.1))
    eq_(result, [TEST_DATA[1]])


def test_min_lift():
    """
    Filter by minimum lift.
    """
    result = list(filter_ordered_statistics(
        TEST_DATA, min_confidence=0.0, min_lift=0.6))
    eq_(result, [TEST_DATA[0]])



"""
Tests for apyori.create_next_candidates.
"""

from nose.tools import eq_
from mock import Mock

from apyori import SupportRecord
from apyori import TransactionManager
from apyori import gen_support_records


def test_empty():
    """
    Test for gen_supports_record.
    """
    transaction_manager = Mock(spec=TransactionManager)
    transaction_manager.initial_candidates.return_value = []
    support_records_gen = gen_support_records(transaction_manager, 0.1)
    support_records = list(support_records_gen)
    eq_(support_records, [])


def test_infinite():
    """
    Test for gen_supports_record with no limits.
    """
    transaction_manager = Mock(spec=TransactionManager)
    transaction_manager.initial_candidates.return_value = [
        frozenset(['A']), frozenset(['B']), frozenset(['C'])]
    transaction_manager.calc_support.side_effect = lambda key: {
        frozenset(['A']): 0.8,
        frozenset(['B']): 0.6,
        frozenset(['C']): 0.3,
        frozenset(['A', 'B']): 0.3,
        frozenset(['A', 'C']): 0.2,
    }.get(key, 0.0)
    candidates = {
        2: [
            frozenset(['A', 'B']),
            frozenset(['A', 'C']),
            frozenset(['B', 'C'])
        ],
        3: [
            frozenset(['A', 'B', 'C']),
        ],
    }
    support_records_gen = gen_support_records(
        transaction_manager, 0.3,
        _create_next_candidates=lambda _, length: candidates.get(length))

    support_records = list(support_records_gen)
    eq_(support_records, [
        SupportRecord(frozenset(['A']), 0.8),
        SupportRecord(frozenset(['B']), 0.6),
        SupportRecord(frozenset(['C']), 0.3),
        SupportRecord(frozenset(['A', 'B']), 0.3),
    ])


def test_length():
    """
    Test for gen_supports_record that limits the length.
    """
    transaction_manager = Mock(spec=TransactionManager)
    transaction_manager.initial_candidates.return_value = [
        frozenset(['A']), frozenset(['B']), frozenset(['C'])]
    transaction_manager.calc_support.side_effect = lambda key: {
        frozenset(['A']): 0.7,
        frozenset(['B']): 0.5,
        frozenset(['C']): 0.2,
        frozenset(['A', 'B']): 0.2,
        frozenset(['A', 'C']): 0.1,
    }.get(key, 0.0)
    candidates = {
        2: [
            frozenset(['A', 'B']),
            frozenset(['A', 'C']),
            frozenset(['B', 'C'])
        ],
    }
    support_records_gen = gen_support_records(
        transaction_manager, 0.05, max_length=1,
        _create_next_candidates=lambda _, length: candidates.get(length))

    support_records = list(support_records_gen)
    eq_(support_records, [
        SupportRecord(frozenset(['A']), 0.7),
        SupportRecord(frozenset(['B']), 0.5),
        SupportRecord(frozenset(['C']), 0.2),
    ])



#!/usr/bin/env python

"""
a simple implementation of Apriori algorithm by Python.
"""

import sys
import csv
import argparse
import json
import os
from collections import namedtuple
from itertools import combinations
from itertools import chain


# Meta informations.
__version__ = '1.1.0'
__author__ = 'Yu Mochizuki'
__author_email__ = 'ymoch.dev@gmail.com'


################################################################################
# Data structures.
################################################################################
class TransactionManager(object):
    """
    Transaction managers.
    """

    def __init__(self, transactions):
        """
        Initialize.

        Arguments:
            transactions -- A transaction iterable object
                            (eg. [['A', 'B'], ['B', 'C']]).
        """
        self.__num_transaction = 0
        self.__items = []
        self.__transaction_index_map = {}

        for transaction in transactions:
            self.add_transaction(transaction)

    def add_transaction(self, transaction):
        """
        Add a transaction.

        Arguments:
            transaction -- A transaction as an iterable object (eg. ['A', 'B']).
        """
        for item in transaction:
            if item not in self.__transaction_index_map:
                self.__items.append(item)
                self.__transaction_index_map[item] = set()
            self.__transaction_index_map[item].add(self.__num_transaction)
        self.__num_transaction += 1

    def calc_support(self, items):
        """
        Returns a support for items.

        Arguments:
            items -- Items as an iterable object (eg. ['A', 'B']).
        """
        # Empty items is supported by all transactions.
        if not items:
            return 1.0

        # Empty transactions supports no items.
        if not self.num_transaction:
            return 0.0

        # Create the transaction index intersection.
        sum_indexes = None
        for item in items:
            indexes = self.__transaction_index_map.get(item)
            if indexes is None:
                # No support for any set that contains a not existing item.
                return 0.0

            if sum_indexes is None:
                # Assign the indexes on the first time.
                sum_indexes = indexes
            else:
                # Calculate the intersection on not the first time.
                sum_indexes = sum_indexes.intersection(indexes)

        # Calculate and return the support.
        return float(len(sum_indexes)) / self.__num_transaction

    def initial_candidates(self):
        """
        Returns the initial candidates.
        """
        return [frozenset([item]) for item in self.items]

    @property
    def num_transaction(self):
        """
        Returns the number of transactions.
        """
        return self.__num_transaction

    @property
    def items(self):
        """
        Returns the item list that the transaction is consisted of.
        """
        return sorted(self.__items)

    @staticmethod
    def create(transactions):
        """
        Create the TransactionManager with a transaction instance.
        If the given instance is a TransactionManager, this returns itself.
        """
        if isinstance(transactions, TransactionManager):
            return transactions
        return TransactionManager(transactions)


# Ignore name errors because these names are namedtuples.
SupportRecord = namedtuple( # pylint: disable=C0103
    'SupportRecord', ('items', 'support'))
RelationRecord = namedtuple( # pylint: disable=C0103
    'RelationRecord', SupportRecord._fields + ('ordered_statistics',))
OrderedStatistic = namedtuple( # pylint: disable=C0103
    'OrderedStatistic', ('items_base', 'items_add', 'confidence', 'lift',))


################################################################################
# Inner functions.
################################################################################
def create_next_candidates(prev_candidates, length):
    """
    Returns the apriori candidates as a list.

    Arguments:
        prev_candidates -- Previous candidates as a list.
        length -- The lengths of the next candidates.
    """
    # Solve the items.
    item_set = set()
    for candidate in prev_candidates:
        for item in candidate:
            item_set.add(item)
    items = sorted(item_set)

    def check_subsets(candidate):
        """
        Check if the subsets of a candidate is present
        in the previous candidates.
        """
        candidate_subsets = [
            frozenset(x) for x in combinations(candidate, length - 1)]
        for candidate_subset in candidate_subsets:
            if candidate_subset not in prev_candidates:
                return False
        return True

    # Create candidates.
    next_candidates = []
    for candidate in [frozenset(x) for x in combinations(items, length)]:
        if length > 2 and not check_subsets(candidate):
            continue
        next_candidates.append(candidate)
    return next_candidates


def gen_support_records(transaction_manager, min_support, **kwargs):
    """
    Returns a generator of support records with given transactions.

    Arguments:
        transaction_manager -- Transactions as a TransactionManager instance.
        min_support -- A minimum support (float).

    Keyword arguments:
        max_length -- The maximum length of relations (integer).
    """
    # Parse arguments.
    max_length = kwargs.get('max_length')

    # For testing.
    _create_next_candidates = kwargs.get(
        '_create_next_candidates', create_next_candidates)

    # Process.
    candidates = transaction_manager.initial_candidates()
    length = 1
    while candidates:
        relations = set()
        for relation_candidate in candidates:
            support = transaction_manager.calc_support(relation_candidate)
            if support < min_support:
                continue
            candidate_set = frozenset(relation_candidate)
            relations.add(candidate_set)
            yield SupportRecord(candidate_set, support)
        length += 1
        if max_length and length > max_length:
            break
        candidates = _create_next_candidates(relations, length)


def gen_ordered_statistics(transaction_manager, record):
    """
    Returns a generator of ordered statistics as OrderedStatistic instances.

    Arguments:
        transaction_manager -- Transactions as a TransactionManager instance.
        record -- A support record as a SupportRecord instance.
    """
    items = record.items
    for combination_set in combinations(sorted(items), len(items) - 1):
        items_base = frozenset(combination_set)
        items_add = frozenset(items.difference(items_base))
        confidence = (
            record.support / transaction_manager.calc_support(items_base))
        lift = confidence / transaction_manager.calc_support(items_add)
        yield OrderedStatistic(
            frozenset(items_base), frozenset(items_add), confidence, lift)


def filter_ordered_statistics(ordered_statistics, **kwargs):
    """
    Filter OrderedStatistic objects.

    Arguments:
        ordered_statistics -- A OrderedStatistic iterable object.

    Keyword arguments:
        min_confidence -- The minimum confidence of relations (float).
        min_lift -- The minimum lift of relations (float).
    """
    min_confidence = kwargs.get('min_confidence', 0.0)
    min_lift = kwargs.get('min_lift', 0.0)

    for ordered_statistic in ordered_statistics:
        if ordered_statistic.confidence < min_confidence:
            continue
        if ordered_statistic.lift < min_lift:
            continue
        yield ordered_statistic


################################################################################
# API function.
################################################################################
def apriori(transactions, **kwargs):
    """
    Executes Apriori algorithm and returns a RelationRecord generator.

    Arguments:
        transactions -- A transaction iterable object
                        (eg. [['A', 'B'], ['B', 'C']]).

    Keyword arguments:
        min_support -- The minimum support of relations (float).
        min_confidence -- The minimum confidence of relations (float).
        min_lift -- The minimum lift of relations (float).
        max_length -- The maximum length of the relation (integer).
    """
    # Parse the arguments.
    min_support = kwargs.get('min_support', 0.1)
    min_confidence = kwargs.get('min_confidence', 0.0)
    min_lift = kwargs.get('min_lift', 0.0)
    max_length = kwargs.get('max_length', None)

    # Check arguments.
    if min_support <= 0:
        raise ValueError('minimum support must be > 0')

    # For testing.
    _gen_support_records = kwargs.get(
        '_gen_support_records', gen_support_records)
    _gen_ordered_statistics = kwargs.get(
        '_gen_ordered_statistics', gen_ordered_statistics)
    _filter_ordered_statistics = kwargs.get(
        '_filter_ordered_statistics', filter_ordered_statistics)

    # Calculate supports.
    transaction_manager = TransactionManager.create(transactions)
    support_records = _gen_support_records(
        transaction_manager, min_support, max_length=max_length)

    # Calculate ordered stats.
    for support_record in support_records:
        ordered_statistics = list(
            _filter_ordered_statistics(
                _gen_ordered_statistics(transaction_manager, support_record),
                min_confidence=min_confidence,
                min_lift=min_lift,
            )
        )
        if not ordered_statistics:
            continue
        yield RelationRecord(
            support_record.items, support_record.support, ordered_statistics)


################################################################################
# Application functions.
################################################################################
def parse_args(argv):
    """
    Parse commandline arguments.

    Arguments:
        argv -- An argument list without the program name.
    """
    output_funcs = {
        'json': dump_as_json,
        'tsv': dump_as_two_item_tsv,
    }
    default_output_func_key = 'json'

    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-v', '--version', action='version',
        version='%(prog)s {0}'.format(__version__))
    parser.add_argument(
        'input', metavar='inpath', nargs='*',
        help='Input transaction file (default: stdin).',
        type=argparse.FileType('r'), default=[sys.stdin])
    parser.add_argument(
        '-o', '--output', metavar='outpath',
        help='Output file (default: stdout).',
        type=argparse.FileType('w'), default=sys.stdout)
    parser.add_argument(
        '-l', '--max-length', metavar='int',
        help='Max length of relations (default: infinite).',
        type=int, default=None)
    parser.add_argument(
        '-s', '--min-support', metavar='float',
        help='Minimum support ratio (must be > 0, default: 0.1).',
        type=float, default=0.1)
    parser.add_argument(
        '-c', '--min-confidence', metavar='float',
        help='Minimum confidence (default: 0.5).',
        type=float, default=0.5)
    parser.add_argument(
        '-t', '--min-lift', metavar='float',
        help='Minimum lift (default: 0.0).',
        type=float, default=0.0)
    parser.add_argument(
        '-d', '--delimiter', metavar='str',
        help='Delimiter for items of transactions (default: tab).',
        type=str, default='\t')
    parser.add_argument(
        '-f', '--out-format', metavar='str',
        help='Output format ({0}; default: {1}).'.format(
            ', '.join(output_funcs.keys()), default_output_func_key),
        type=str, choices=output_funcs.keys(), default=default_output_func_key)
    args = parser.parse_args(argv)

    args.output_func = output_funcs[args.out_format]
    return args


def load_transactions(input_file, **kwargs):
    """
    Load transactions and returns a generator for transactions.

    Arguments:
        input_file -- An input file.

    Keyword arguments:
        delimiter -- The delimiter of the transaction.
    """
    delimiter = kwargs.get('delimiter', '\t')
    for transaction in csv.reader(input_file, delimiter=delimiter):
        if not transaction:
            yield ['']
        else:
            yield transaction


def dump_as_json(record, output_file):
    """
    Dump an relation record as a json value.

    Arguments:
        record -- A RelationRecord instance to dump.
        output_file -- A file to output.
    """
    def default_func(value):
        """
        Default conversion for JSON value.
        """
        if isinstance(value, frozenset):
            return sorted(value)
        raise TypeError(repr(value) + " is not JSON serializable")

    converted_record = record._replace(
        ordered_statistics=[x._asdict() for x in record.ordered_statistics])
    json.dump(
        converted_record._asdict(), output_file,
        default=default_func, ensure_ascii=False)
    output_file.write(os.linesep)


def dump_as_two_item_tsv(record, output_file):
    """
    Dump a relation record as TSV only for 2 item relations.

    Arguments:
        record -- A RelationRecord instance to dump.
        output_file -- A file to output.
    """
    for ordered_stats in record.ordered_statistics:
        if len(ordered_stats.items_base) != 1:
            continue
        if len(ordered_stats.items_add) != 1:
            continue
        output_file.write('{0}\t{1}\t{2:.8f}\t{3:.8f}\t{4:.8f}{5}'.format(
            list(ordered_stats.items_base)[0], list(ordered_stats.items_add)[0],
            record.support, ordered_stats.confidence, ordered_stats.lift,
            os.linesep))


def main(**kwargs):
    """
    Executes Apriori algorithm and print its result.
    """
    # For tests.
    _parse_args = kwargs.get('_parse_args', parse_args)
    _load_transactions = kwargs.get('_load_transactions', load_transactions)
    _apriori = kwargs.get('_apriori', apriori)

    args = _parse_args(sys.argv[1:])
    transactions = _load_transactions(
        chain(*args.input), delimiter=args.delimiter)
    result = _apriori(
        transactions,
        max_length=args.max_length,
        min_support=args.min_support,
        min_confidence=args.min_confidence)
    for record in result:
        args.output_func(record, args.output)


if __name__ == '__main__':
    main()



"""
Tests for apyori.apriori.
"""

from nose.tools import eq_
from nose.tools import raises

from mock import Mock

from apyori import TransactionManager
from apyori import SupportRecord
from apyori import RelationRecord
from apyori import OrderedStatistic
from apyori import apriori


def test_empty():
    """
    Test for empty data.
    """
    transaction_manager = Mock(spec=TransactionManager)
    dummy_return = OrderedStatistic(
        frozenset(['A']), frozenset(['B']), 0.1, 0.7)
    def gen_support_records(*args, **kwargs): # pylint: disable=unused-argument
        """ Mock for apyori.gen_support_records. """
        return iter([])

    def gen_ordered_statistics(*_):
        """ Mock for apyori.gen_ordered_statistics. """
        yield dummy_return

    def filter_ordered_statistics(*_):
        """ Mock for apyori.gen_ordered_statistics. """
        yield dummy_return

    result = list(apriori(
        transaction_manager,
        _gen_support_records=gen_support_records,
        _gen_ordered_statistics=gen_ordered_statistics,
        _filter_ordered_statistics=filter_ordered_statistics,
    ))
    eq_(result, [])


def test_filtered():
    """
    Test for filtered data.
    """
    transaction_manager = Mock(spec=TransactionManager)
    dummy_return = OrderedStatistic(
        frozenset(['A']), frozenset(['B']), 0.1, 0.7)
    def gen_support_records(*args, **kwargs): # pylint: disable=unused-argument
        """ Mock for apyori.gen_support_records. """
        yield dummy_return

    def gen_ordered_statistics(*_):
        """ Mock for apyori.gen_ordered_statistics. """
        yield dummy_return

    def filter_ordered_statistics(*args, **kwargs): # pylint: disable=unused-argument
        """ Mock for apyori.gen_ordered_statistics. """
        return iter([])

    result = list(apriori(
        transaction_manager,
        _gen_support_records=gen_support_records,
        _gen_ordered_statistics=gen_ordered_statistics,
        _filter_ordered_statistics=filter_ordered_statistics,
    ))
    eq_(result, [])


def test_normal():
    """
    Test for normal data.
    """
    transaction_manager = Mock(spec=TransactionManager)
    min_support = 0.1
    min_confidence = 0.1
    min_lift = 0.5
    max_length = 2
    support_record = SupportRecord(frozenset(['A', 'B']), 0.5)
    ordered_statistic1 = OrderedStatistic(
        frozenset(['A']), frozenset(['B']), 0.1, 0.7)
    ordered_statistic2 = OrderedStatistic(
        frozenset(['A']), frozenset(['B']), 0.3, 0.5)

    def gen_support_records(*args, **kwargs):
        """ Mock for apyori.gen_support_records. """
        eq_(args[1], min_support)
        eq_(kwargs['max_length'], max_length)
        yield support_record

    def gen_ordered_statistics(*_):
        """ Mock for apyori.gen_ordered_statistics. """
        yield ordered_statistic1
        yield ordered_statistic2

    def filter_ordered_statistics(*args, **kwargs):
        """ Mock for apyori.gen_ordered_statistics. """
        eq_(kwargs['min_confidence'], min_confidence)
        eq_(kwargs['min_lift'], min_lift)
        eq_(len(list(args[0])), 2)
        yield ordered_statistic1

    result = list(apriori(
        transaction_manager,
        min_support=min_support,
        min_confidence=min_confidence,
        min_lift=min_lift,
        max_length=max_length,
        _gen_support_records=gen_support_records,
        _gen_ordered_statistics=gen_ordered_statistics,
        _filter_ordered_statistics=filter_ordered_statistics,
    ))
    eq_(result, [RelationRecord(
        support_record.items, support_record.support, [ordered_statistic1]
    )])


@raises(ValueError)
def test_invalid_support():
    """
    An invalid support.
    """
    transaction_manager = Mock(spec=TransactionManager)
    list(apriori(transaction_manager, min_support=0.0))



#!/usr/bin/env python

"""
Setting up program for Apyori.
"""

import apyori
import setuptools

setuptools.setup(
    name='apyori',
    description='Simple Apriori algorithm Implementation.',
    long_description=open('README.rst').read(),
    version=apyori.__version__,
    author=apyori.__author__,
    author_email=apyori.__author_email__,
    url='https://github.com/ymoch/apyori',
    py_modules=['apyori'],
    test_suite='nose.collector',
    tests_require=['nose', 'mock'],
    entry_points={
        'console_scripts': [
            'apyori-run = apyori:main',
        ],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)
